 clc;
clear all;

global data X Y

%------------------------------------------------------------------
% This code generates Figures 2 and 3 from the paper "Crude Substitution:
% the cyclical dynamics of oil prices and the skill premium.
% The methodology is that described in section 2.2 of that paper.
% Written by Pedro Silos
%------------------------------------------------------------------

% var_data is a text file that has data on CPI, RGDP, oil prices, 
% the skill premium, and Hoover-Perez dates - all in levels - from 
% 1963-2004.

load var_data.txt;

spec = 1; % specification choice: 1=4 variable (CPI, RGDP, OP, SP) , 
          % 2=2 variable (OP, SP).
if (spec==1) 
data = var_data(:,1:4);
else
data = var_data(:,3:4);
end
perez = var_data(:,5);

data=log(data); % the VAR is run in logs.

% Create X matrix (includes one constant, the Hoover-Perez Dummy variable 
% for oil shocks, and lags of the endogenous variables.


nlags = 1; % # of lags
nvars = cols(data); % # of variables
smpl = nlags+1;
smplend = rows(data);
trend = 1:1:smplend-smpl+1;
trend2 = trend.^2;

X = [ones(smplend-smpl+1,1) perez(smpl:rows(perez)) ...
     data(smpl-1:rows(data)-1,:) ];

Y = data(smpl:rows(data),:); 
 
% OLS estimates of the VAR.

bhat = inv(X'*X)*X'*Y; 
S = (Y-X*bhat)'*(Y-X*bhat);

yfit = X*bhat; 

maxhoriz=15; % Number of years to plot the impulse response
Mboot = 500; % Number of bootstrap draws
impulse_boot = zeros(maxhoriz,cols(data),Mboot); % vector to store
                                                 % all simuls of IRFS

eps = (Y-X*bhat); 
index_vec= 1:1:rows(eps); % index vector to sample with replacement.
ysim = zeros(rows(Y),cols(Y),Mboot); % vector to keep 

%----------------------------------------------------------------
% Beginning of the bootstrap loop to compute IRFS 
%----------------------------------------------------------------

for i=1:Mboot
xdata = zeros(rows(X),cols(X));
replac = randsample(index_vec,rows(eps),true);
xinit = [1 perez(1) data(1,:)];

xdata(1,:) = xinit;
k=2;
for j=1:rows(Y)
ysim(j,:,i) = xinit*bhat+eps(replac(j),:);
xinit = [1 X(j,2) ysim(j,:,i)];
xdata(j+1,:) = xinit;
end
xdata = xdata(1:rows(Y),:);
bhat2 = inv(xdata'*xdata)*xdata'*ysim(:,:,i);
Amat2 = bhat2(3:3+nvars-1,:)';
Dmat2 = bhat2(2,:);
impulse_boot(1,:,i) = Dmat2;

for jj=2:maxhoriz
impulse_boot(jj,:,i) = Amat2*impulse_boot(jj-1,:,i)';
end
end
clear Amat2 Dmat2 bhat2 xdata replac; 

 
in1 = 0.17; % constant to compute the in1-th percentile and the 
            % (1-in1)-th percentile

% storage for the median and the two bands
% for each variable

impest = zeros(maxhoriz,3); 
impest_oil = zeros(maxhoriz,3);
impest_gdp = zeros(maxhoriz,3);
impest_cpi = zeros(maxhoriz,3);

for j=1:maxhoriz;
    
    if (spec==1) 
    xvec = sort(impulse_boot(j,4,:));
    xvec_oil = sort(impulse_boot(j,3,:));
    xvec_gdp = sort(impulse_boot(j,2,:));
    xvec_cpi = sort(impulse_boot(j,1,:));
    impest(j,1) = xvec(:,:,floor(in1*Mboot));
    impest(j,2) = xvec(:,:,floor(0.5*Mboot));
    impest(j,3) = xvec(:,:,floor((1-in1)*Mboot));
    impest_oil(j,1) = xvec_oil(:,:,floor(in1*Mboot));
    impest_oil(j,2) = xvec_oil(:,:,floor(0.5*Mboot));
    impest_oil(j,3) = xvec_oil(:,:,floor((1-in1)*Mboot)); 
    impest_gdp(j,1) = xvec_gdp(:,:,floor(in1*Mboot));
    impest_gdp(j,2) = xvec_gdp(:,:,floor(0.5*Mboot));
    impest_gdp(j,3) = xvec_gdp(:,:,floor((1-in1)*Mboot)); 
    impest_cpi(j,1) = xvec_cpi(:,:,floor(in1*Mboot));
    impest_cpi(j,2) = xvec_cpi(:,:,floor(0.5*Mboot));
    impest_cpi(j,3) = xvec_cpi(:,:,floor((1-in1)*Mboot)); 
    else
        
    xvec = sort(impulse_boot(j,2,:));    
    xvec2 = sort(impulse_boot(j,1,:));    
    impest_oil(j,1) = xvec2(:,:,floor(in1*Mboot));
    impest_oil(j,2) = xvec2(:,:,floor(0.5*Mboot));
    impest_oil(j,3) = xvec2(:,:,floor((1-in1)*Mboot));
    impest_sp(j,1) = xvec(:,:,floor(in1*Mboot));
    impest_sp(j,2) = xvec(:,:,floor(0.5*Mboot));
    impest_sp(j,3) = xvec(:,:,floor((1-in1)*Mboot));
    end
    
    

end

zero_vec = zeros(rows(impest),1);

%--------------------------------------------
% PLOTS!
%--------------------------------------------

 
if (spec==1) % Four graph plot (Spec=1)
subplot(2,2,1);
plot(zero_vec);
hold on
plot(impest_oil(:,1),'--')
hold on
plot(impest_oil(:,2),'LineWidth',2)
hold on
plot(impest_oil(:,3),'--')
xlabel('Years')
ylabel('Response/Oil Prices')
subplot(2,2,2);
plot(zero_vec);
hold on
plot(impest(:,1),'--')
hold on
plot(impest(:,2),'LineWidth',2)
hold on
plot(impest(:,3),'--')
xlabel('Years')
ylabel('Response/Skill Premium')
subplot(2,2,3);
plot(zero_vec);
hold on
plot(impest_gdp(:,1),'--')
hold on
plot(impest_gdp(:,2),'LineWidth',2)
hold on
plot(impest_gdp(:,3),'--')
xlabel('Years')
ylabel('Response/Output')
subplot(2,2,4);
plot(zero_vec);
hold on
plot(impest_cpi(:,1),'--')
hold on
plot(impest_cpi(:,2),'LineWidth',2)
hold on
plot(impest_cpi(:,3),'--')
xlabel('Years')
ylabel('Response/CPI')

else  % Two-graph plot (Spec=2)
    
subplot(2,1,1);
zero_vec = zeros(rows(impest_oil),1);
plot(zero_vec);
hold on
plot(impest_oil(:,1),'--')
hold on
plot(impest_oil(:,2),'LineWidth',2)
hold on
plot(impest_oil(:,3),'--')
xlabel('Years')
ylabel('Response / Oil Prices')
subplot(2,1,2);
zero_vec = zeros(rows(impest_oil),1);
plot(zero_vec);
hold on
plot(impest_sp(:,1),'--')
hold on
plot(impest_sp(:,2),'LineWidth',2)
hold on
plot(impest_sp(:,3),'--')
xlabel('Years')
ylabel('Response / Skill Premium')
end
