
clc;
clear all
tstart=cputime;

global Paramdraw Omegadraw H0draw STATE1 STATE2 covdata ydata xdata

% This paper estimates the model described in Section 3 of the paper,
% "Crude Substitution: The Cyclical Dynamics of Oil Prices and the Skill
% Premium".  There are several files that need to be in the same directory,
% but this is the main program, which calls all the others.
% As described in the paper, the procedure has two parts:
% First, estimate nu by OLS.
% Second, all the other parameters.
% The second part is done in three blocks: sampling of the latent variables
% (gtan.m), sampling of paramters of the measurement equations
% (sample_meas.m), and finally sampling the parameters of the transition
% equation (sample_tran.m).
% All the necessary files should be included in the journal site.
% The algorithm is essentially the same as that described in Polgreen, L.
% and Silos, P. (2006) "Capital-Skill Complementarity and Inequaality: A
% Sensitivity Analysis", Atlanta Fed Working Paper (Technical Appendix).

% WARNING(S): 1.-The algorithm is tailor-made for this particular problem.
% Although it can be adapted without much difficulty to similar problems,
% using different depreciation rates, or different datasets, might cause it
% to not work properly.  
% 2.- The algorithm is based on simulation of random
% variables. As such, it is subject to some variation in its parameter
% estimates. Hence, the estimated posteriors for sigma, rho, etc... might
% not be exactly those presented in the article tables. The variations
% should not be large, and in no case should they modify the conclusions in
% the paper.

% Please address all concerns,comments,suggestions, etc.. to
% pedrosilos@gmail.com

rand('state',1456);
randn('state',1456);

% Estimation of "\nu" by OLS (See section3)
load dataforestimatingnu.txt;
nudata = dataforestimatingnu;
y = nudata(:,1);
x = nudata(:,2);
[b_nu,bint,res]=regress(y,x);  % b_nu is beta
se = sum(res.^2)/(rows(y));
sb = sqrt( se*inv(x'*x) ); % std error of beta

% Load data for the second part of the estimation.

load data1963_2004.txt;
 
data=data1963_2004; 
 
% The model is composed of 2 unobserved state variables, phi_u(t) and phi_s(t) and three measurable equations.
% the order of the parameters is (sigma, rho,mu,lambda,alpha, hu(0),sig2(w),sig2(eps), deltas, delte, mu, nu)
% Mu and nu have added to KORV's original estimation to take into account
% for the presence of energy:  So the production function is the following
% GDP =
% Ks^alpha*(mu*u^sigma+(1-mu)*(lambda*(xi*Ke^nu+(1-xi)E^nu)^(rho/nu)+(1-lam
% bda)st^rho)^(sigma/rho))^(1-alpha)/sigma
% there are also 6 covariates if the following order (capital equip data, capital struct data, labor hours (skilled),...
% labor hours (unskilled), equip prices, Energy use);

% The order of the unobserved state is always: efficiency of unskilled, efficiency of skilled.
%
% Sigma is the (2x2) covariance matrix of the state equations.
%
% Omega is the (3x3) covariance matrix of the 3 measurement equations.
%
% Transition equation: h(t)=F+v(t)
% Measurable equation: y(t)=f(parameters,covariates,h(t))+epsilon(t);

ToTdata=data;
T = rows(ToTdata);
 

% get data and covariates

covdata=[ToTdata(2:T,3) ToTdata(2:T,2) ToTdata(2:T,6) ToTdata(2:T,5) ToTdata(1:T-1,4) ToTdata(2:T,10)]; 
ydata=[ToTdata(2:T,9) ToTdata(2:T,7)./ToTdata(2:T,8) ToTdata(2:T,4)./covdata(:,5)];

T=rows(ydata);

 
m_sigma = 0.575;
s_sigma=0.25;

m_rho = -.76;
s_rho=0.25;

m_mu=0.5;
s_mu=0.20;

m_lambda=0.5;
s_lambda=0.20;

m_alpha=0.11;
s_alpha=0.005;

m_gamma1=0.05;
s_gamma1=0.1;

m_gamma2=0.07;
s_gamma2=0.1;

m_phi=1.3;
s_phi=0.15;

m_phi2=2;
s_phi2=0.15;

m_xi = 0.5;
s_xi = 0.1;

a_omega=0.3;
b_omega=0.01;

a_sigma=0.4;
b_sigma=0.01;

deltas=0.05;
deltae=0.125;


% start Gibbs Sampling loop


Nmc=500000; % number of MC draws


% Matrices for storage
STATE1=zeros(rows(ydata),Nmc);
STATE2=zeros(rows(ydata),Nmc);
Paramdraw=zeros(Nmc,12);
Omegadraw=zeros(Nmc,3);
Sigmadraw=zeros(Nmc,1);
H0draw=zeros(Nmc,2);

% get initial value from above;
init_state(:,1)=ones(T,1)*m_phi+normrnd(0,sqrt(a_sigma*b_sigma),rows(ydata),1);
init_state(:,2)=ones(T,1)*m_phi2+normrnd(0,sqrt(a_sigma*b_sigma),rows(ydata),1);
STATE1(:,1)=init_state(:,1);
STATE2(:,1)=init_state(:,2);

H0=[m_phi m_phi2];
H0draw(1,:)=H0;

initparams=[ 0.5  -0.9  0.5  0.5    0.11    2.3    0    0.04    0.00500    0.1250  0.2 b_nu]; 

Sigma=[initparams(8) 0; 0 initparams(8)];
Omega=[3d-4 0 0; 0 3d-4 0; 0 0 1d-3];
Paramdraw(1,:)=initparams;
Omegadraw(1,:)=diag(Omega)';
HOdraw(1,:)=H0;

% -----------------------------------------------------------------------
% -----------------------------------------------------------------------
% -- Big Monte Carlo loop
% -----------------------------------------------------------------------
% -----------------------------------------------------------------------

for k=2:Nmc;
 
Sigmaparam=diag(1d-3*[1d-2 1d-2 1d-2 1d-2 1d-4 1d-1]) ;
Sigmaparam=diag(1d-3*[1d-2 1d-1 1d-2 1d-2 1d-4 1d-1]) ;
Sigmaomega=diag(8d-4*[1d-5 1d-7 1d-5]);


 if (Paramdraw(k-1,11)>0.98)
    Sigmaparam=diag(1d-5*[0.7 0.7 0.7 0.7 1d-4 5d-5]) ;
     Sigmaparam(6,6) = 1d-6;
 end
 
 
if min(Omegadraw(k-1,:))<1d-5;
Sigmaparam=diag(2d-7*[0.1 0.1 1d-3 1d-3 1d-4 1d-3]) ;
Sigmaparam(6,6) = 1d-09;
Sigmaomega=diag([1d-11 1d-11 1d-10]);
end

% Block 1)
gtan; % sampling latent variabls (geweke-tanizaki step).
% Block 2) 
sample_meas; % sampling parameters of the measurement equations.
% Block 3)
sample_tran; % sampling the parameters of the transition equation.

Paramdraw(k,:)

k=k+1;
end

% -----------------------------------------------------------------------
% -----------------------------------------------------------------------
% -- End of big loop
% -----------------------------------------------------------------------
% -----------------------------------------------------------------------
 
% measuring time of finishing
tend=cputime;

save Paramdraw Paramdraw
save Omegadraw Omegadraw
save H0draw H0draw
save STATE1 STATE1
save STATE2 STATE2

%-----------------------------------------------------------------------
%-----------------------------------------------------------------------
%-----------------------------------------------------------------------


burnin = 200000;
% Discard the first 20% of draws
Paramdrawnew = Paramdraw(burnin+1:rows(Paramdraw),:);
oilprices = log(data(2:rows(data),12)); %load data on oil prices (lose one obs)

% some storage stuff
corrdt = zeros(rows(Paramdrawnew),1);
corrbp = zeros(rows(Paramdrawnew),1);
corrhp = zeros(rows(Paramdrawnew),1);
 

time = 1:1:rows(STATE1);
time = time';
for i=1:rows(Paramdrawnew);
    param = Paramdrawnew(i,:);
    param(12) = randn(1,1)*sb - b_nu;
    xdata = [mean(H0draw(burnin+1:rows(Paramdraw),1))*ones(rows(covdata),1) mean(H0draw(burnin+1:rows(Paramdraw),2))*ones(rows(covdata),1)];
    spmodel = log(measfun2(param,covdata,xdata)); 
    gdp_model = log(output_oil(param,covdata,xdata));
    z=corrcoef(bpass(spmodel,3,35),bpass(oilprices,3,35));
    corrbp(i) = z(1,2);
    z=corrcoef(bpass(spmodel,3,35),bpass(gdp_model,3,35));
    corrbp(i) = z(1,2)
    z=corrcoef(spmodel-hpfilter(spmodel,100),oilprices-hpfilter(oilprices,100));
    corrhp(i) = z(1,2);
    b=regress(spmodel,[ones(rows(xdata),1) time]);
    spdt = spmodel-b(1)-b(2)*time;
    b=regress(oilprices,[ones(rows(xdata),1) time]);
    opdt = oilprices-b(1)-b(2)*time;
    b=regress(gdp_model,[ones(rows(xdata),1) time]);
    gdpdt = gdp_model-b(1)-b(2)*time;
    z=corrcoef(spdt,opdt);
    corrdt(i) =z(1,2);    
    i
end

% Display some correlations
disp('Corr. between Oil prices and the skill premium (post draws)')
disp('Exp. Detrend')
mean(corrdt)
std(corrdt)
disp('BP filt')
mean(corrbp)
std(corrbp)
disp('HP Filt')
mean(corrhp)
std(corrhp)


 
 


